#ifndef MDD_PROCESSORBASE_H
#define MDD_PROCESSORBASE_H

#include <list>
#include "Input.h"
#include "IProcessor.h"
#include "HandlerModule.h"

namespace mdd {
    class ProcessorBase : public IProcessor{
    private:
        std::string _prefix;
        std::string _type;
        int _appendix;
        std::vector<Input> _processor_inputs;
        std::vector<std::shared_ptr<Output>> _processor_outputs;

        std::vector<HandlerModule> _module_inputs;
        std::vector<HandlerModule> _module_outputs;
        std::vector<std::shared_ptr<IModule>> _modules;
    public:
        void setType(std:: string type) override;
        std::string getType() override;
        std::string addModule(std::shared_ptr<IModule> module) override ;
        std::string addProcesorInput(const std::string& type, int  const json& value,
                     const std::function<bool(const json&)>& verification = [](
                             const json&) { return true; });
        std::string addModuleInput(std::string moduleHandler, std::string inputHandler);
        std::string addModuleOutput(std::string moduleHandler, std::string outputHandler);
        std::string addProcessorOutput(const std::string& type, const json& initial);
        json& setInputDefaultValue(std::string input_id) override;
        json& setOutputValue(int handle);
        const json& getInputValue(int handle) override;
        bool connectInput(std::string input_id, std::shared_ptr<IOutput> output) override;
        std::vector<std::string> getInputs() override;
        std::vector<std::string> getOutputs() override;
        std::shared_ptr<IOutput> getOutput(std::string output_id) override;
        std::vector<std::string> getModules() override;
        std::shared_ptr<IModule> getModule(std::string module_id) override;
    };
}
#endif
