#include "ModuleBase.h"
#include <algorithm>

namespace mdd {

    void ModuleBase::setType(std:: string type){
        _type=type;
        std::string new_prefix =  _prefix + "/" + _type + std::to_string(_appendix);
        for (auto &input  : _inputs){
            input.setPrefix(new_prefix);
        }
        for (auto &output  : _outputs){
            output->setPrefix(new_prefix);
        }
    }
    std::string ModuleBase::getType(){
        return _type;
    }

    json& ModuleBase::setInputDefaultValue(std::string input_id) {
        auto handler = std::find(_inputs.begin(), _inputs.end(), input_id);
        if (handler != _inputs.end()){
            return handler->value;
        }
    }

    json& ModuleBase::setInputDefaultValue(int handle) {
        return _inputs[handle].value;
    }

    const json& ModuleBase::getInputValue(int handle) {
        if (_inputs[handle].output == nullptr)
            return _inputs[handle].value;

        return _inputs[handle].output->getValue();

    }

    json& ModuleBase::setOutputValue(int handle) {
        static json empty;
        if (_outputs.size() <= handle)
            return empty;

        return _outputs[handle]->getValueInternal();
    }


    std::vector<std::string> ModuleBase::getInputs() {
        std::vector<std::string> ret;
        for (auto &input  : _inputs) {
            ret.push_back(input.getType());
        }
        return ret;
    }

    std::vector<std::string> ModuleBase::getInputIDs(){
        std::vector<std::string> ret;
        for (auto &input  : _inputs) {
            ret.push_back(input.getID());
        }
        return ret;
    }

    std::vector<std::string> ModuleBase::getOutputs() {
        std::vector<std::string> ret;
        for (auto &output  : _outputs) {
            ret.push_back(output->getType());
        }
        return ret;
    }

    std::vector<std::string> ModuleBase::getOutputIDs(){
        std::vector<std::string> ret;
        for (auto &output  : _outputs) {
            ret.push_back(output->getID());
        }
        return ret;
    }

    std::string ModuleBase::addInput(const std::string& type, int appendix , const json& value,
                             const std::function<bool(const json&)>& verification) {
        _inputs.emplace_back(Input(type, appendix, value, verification));
        _inputs[_inputs.size() - 1].setPrefix(getID());
        return _inputs[_inputs.size() - 1].getID();
    }

    bool ModuleBase::connectInput(std::string input_id, std::shared_ptr<IOutput> output) {
        auto handler = std::find(_inputs.begin(),_inputs.end(), input_id);
        if (handler != _inputs.end()){
            if (handler->verification(output->getValue())) {
                handler->output = output;
                return true;
            }
        }
        return false;
    }

    std::shared_ptr<IOutput> ModuleBase::getOutput(std::string output_id) {
        auto handler = std::find(_outputs.begin(),_outputs.end(), output_id);
        if (handler != _outputs.end()) {
            return *handler;
        }
        return nullptr;
    }

    std::string ModuleBase::addOutput(const std::string& type, int appendix, const json& initial) {
        _outputs.push_back(std::make_shared<Output>( type, initial));
        _outputs[_outputs.size() - 1]->setPrefix(getID());
        return  _outputs[_outputs.size()-1]->getID();
    }

    std::string ModuleBase::getID(){
        return _prefix + "/" + _type + std::to_string(_appendix);
    }

    std::string ModuleBase::setPrefix(std::string prefix){
        _prefix = prefix;
        std::string new_prefix =  getID();
        for (auto &input  : _inputs){
            input.setPrefix(new_prefix);
        }
        for (auto &output  : _outputs){
            output->setPrefix(new_prefix);
        }
        return new_prefix;
    }

    std::string ModuleBase::setAppendix(int appendix){
        _appendix = appendix;
        std::string new_prefix =  getID();
        for (auto &input  : _inputs){
            input.setPrefix(new_prefix);
        }
        for (auto &output  : _outputs){
            output->setPrefix(new_prefix);
        }
        return new_prefix;
    }

}